/* rexx */

/*
 * Steward Test Suite Generator
 *
 */

uppercase = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
lowercase = 'abcdefghijklmnopqrstuvwxyz'
Env = 'OS2ENVIRONMENT'
FALSE = 0
TRUE = 1

/* Variables normally read from the configuration file */
/* These values are provided as defaults only */
HomeDir = 'c:'
LogDir = 'c:'
ListDir = 'c:'

/* The external functions we need */
call RxFuncAdd 'SysFileDelete', 'RexxUtil', 'SysFileDelete'
call RxFuncAdd 'SysSleep', 'RexxUtil', 'SysSleep'

/* Change both these users to usernames on your mail server */
User = 'joeuser'
User2 = 'janeuser'
/* Change this domain to your mail server */
Domain = 'hops.ag.utk.edu'
/* If you use a different name from steward, put it here */
WhoAmI = 'steward'
/* The account used for testing. It should not have any members */
ListName = 'test'
OldDir = '.'

LogFile = 'test.log'
rc = SysFileDelete(LogFile)
rc = stream(LogFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open log file: test.log'
  exit
  end

say 'Steward Test Suite Generator'

call Init  /* Do some setup stuff */
OldDir = directory()
call directory(HomeDir)

say ''
say 'First series -- Admin Functions/Open List'
rc = lineout(LogFile, 'First series -- Admin Functions/Open List', )
Moderated = 0
NoList = 0
SubscribePolicy = 'open'
OpenPosting = 0
call ReWriteCf

say '1. Subscribe request'
rc = lineout(LogFile, '1. Subscribe request', )
say ''

MsgFile = '1.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User'@'Domain
MsgTo = WhoAmI
call WriteHeaders
rc = lineout(MsgFile, 'subscribe' ListName User'@'Domain, )
rc = lineout(MsgFile, 'Marker: 1. subscribe request', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 1. subscribe request', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
say ''
say '2. Subscribe request - email addresses do not match'
rc = lineout(LogFile, '2. Subscribe request - email addresses do not match', )
say ''

MsgFile = '2.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'subscribe' ListName User2'@'Domain, )
rc = lineout(MsgFile, 'Marker: 2. subscribe request, email mismatch', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 2. subscribe request, email mismatch', )
rc = lineout(LogFile, 'Sender should receive error message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
say ''
say '3. Subscribe request - email addresses do not match with password'
rc = lineout(LogFile, '3. Subscribe request - email addresses do not match with password',)
say ''

MsgFile = '3.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'approved: test-pass', )
rc = lineout(MsgFile, 'subscribe' ListName User2'@'Domain, )
rc = lineout(MsgFile, 'Marker: 3. subscribe request, email mismatch with password', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 3. subscribe request, email mismatch with password', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, 'New member should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User2'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '4. Unsubscribe request'
rc = lineout(LogFile, '4. Unsubscribe request', )
say ''

MsgFile = '4.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'unsubscribe' ListName User'@'Domain, )
rc = lineout(MsgFile, 'Marker: 4. unsubscribe request', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 4. unsubscribe request', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '5. Unsubscribe request - email addresses do not match'
rc = lineout(LogFile, '5. Unsubscribe request - email addresses do not match', )
say ''

MsgFile = '5.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'unsubscribe' ListName User2'@'Domain, )
rc = lineout(MsgFile, 'Marker: 5. unsubscribe request, email mismatch', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 5. unsubscribe request', )
rc = lineout(LogFile, 'Sender should receive error message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '6. Unsubscribe request - email addresses do not match with password'
rc = lineout(LogFile, '6. Unsubscribe request - email addresses do not match with password', )
say ''

MsgFile = '6.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'approved: test-pass', )
rc = lineout(MsgFile, 'unsubscribe' ListName User2'@'Domain, )
rc = lineout(MsgFile, 'Marker: 6. unsubscribe request, email mismatch with password', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 6. unsubscribe request', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, 'New ex-member should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User2'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '7. Lists request'
rc = lineout(LogFile, '7. Lists request', )
say ''

MsgFile = '7.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'lists', )
rc = lineout(MsgFile, 'Marker: 7. lists', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 7. lists request', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '8. Help request'
rc = lineout(LogFile, '8. Help request', )
say ''

MsgFile = '8.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'help', )
rc = lineout(MsgFile, 'Marker: 8. help', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 8. help request', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '9. Null/blank request'
rc = lineout(LogFile, '9. Null/blank request', )
say ''

MsgFile = '9.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, '', )
rc = lineout(MsgFile, 'Marker: 9. null/blank', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 9. null/blank request', )
rc = lineout(LogFile, 'Sender should receive ok reply with help.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
say ''
say 'Second series -- Admin Functions/Closed List'
rc = lineout(LogFile, 'Second series -- Admin Functions/Closed List', )
Moderated = 0
NoList = 0
SubscribePolicy = 'closed'
OpenPosting = 0
call ReWriteCf

say ''
say '10. Subscribe request'
rc = lineout(LogFile, '10. Subscribe request', )
say ''

MsgFile = '10.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'subscribe' ListName User'@'Domain, )
rc = lineout(MsgFile, 'Marker: 10. subscribe request', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 10. subscribe request', )
rc = lineout(LogFile, 'Sender should receive acknowledgement.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, 'List owner should receive subscription approval request.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '11. Subscribe request - email addresses do not match'
rc = lineout(LogFile, '11. Subscribe request - email addresses do not match', )
say ''

MsgFile = '11.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'subscribe' ListName User2'@'Domain, )
rc = lineout(MsgFile, 'Marker: 11. subscribe request, email mismatch', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 11. subscribe request, email mismatch', )
rc = lineout(LogFile, 'Sender should receive error message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '12. Subscribe request - email addresses do not match with password'
rc = lineout(LogFile, '12. Subscribe request - email addresses do not match with password', )
say ''

MsgFile = '12.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'approved: test-pass', )
rc = lineout(MsgFile, 'subscribe' ListName User2'@'Domain, )
rc = lineout(MsgFile, 'Marker: 12. subscribe request, email mismatch with password', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 12. subscribe request, email mismatch with password', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, 'New member should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User2'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '13. Unsubscribe request'
rc = lineout(LogFile, '13. Unsubscribe request', )
say ''

MsgFile = '13.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'unsubscribe' ListName User'@'Domain, )
rc = lineout(MsgFile, 'Marker: 13. unsubscribe request', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 13. unsubscribe request', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '14. Unsubscribe request - email addresses do not match'
rc = lineout(LogFile, '14. Unsubscribe request - email addresses do not match', )
say ''

MsgFile = '14.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'unsubscribe' ListName User2'@'Domain, )
rc = lineout(MsgFile, 'Marker: 14. unsubscribe request, email mismatch', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 14. unsubscribe request', )
rc = lineout(LogFile, 'Sender should receive error message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '15. Unsubscribe request - email addresses do not match with password'
rc = lineout(LogFile, '15. Unsubscribe request - email addresses do not match with password', )
say ''

MsgFile = '15.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'approved: test-pass', )
rc = lineout(MsgFile, 'unsubscribe' ListName User2'@'Domain, )
rc = lineout(MsgFile, 'Marker: 15. unsubscribe request, email mismatch with password', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 15. unsubscribe request', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, 'New ex-member should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User2'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say 'Third series -- Admin Functions/No List'
rc = lineout(LogFile, 'Third series -- Admin Functions/No List', )
Moderated = 0
NoList = 1
SubscribePolicy = 'open'
OpenPosting = 0
call ReWriteCf

say ''
say '16. Lists request'
rc = lineout(LogFile, '16. Lists request', )
say ''

MsgFile = '16.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call WriteHeaders
rc = lineout(MsgFile, 'lists', )
rc = lineout(MsgFile, 'Marker: 16. lists request, no list set', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 16. lists request', )
rc = lineout(LogFile, 'Sender should receive ok reply.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward WhoAmI MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say 'Fourth series -- Message Functions/Open List'
rc = lineout(LogFile, 'Fourth series -- Message Functions/Open List', )
Moderated = 0
NoList = 0
SubscribePolicy = 'open'
OpenPosting = 0
call ReWriteCf
call SetMember(User'@'Domain)

say ''
say '17. Message from member'
rc = lineout(LogFile, '17. Message from member', )
say ''

MsgFile = '17.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from member', )
rc = lineout(MsgFile, 'Marker: 17. Message from member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 17. Message from member.', )
rc = lineout(LogFile, 'List should receive message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
say ''
say '18. Message from non-member'
rc = lineout(LogFile, '18. Message from non-member', )
say ''

MsgFile = '18.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User2'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from non-member', )
rc = lineout(MsgFile, 'Marker: 18. Message from non-member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 18. Message from non-member.', )
rc = lineout(LogFile, 'Sender should receive error message.', )
rc = lineout(LogFile, 'Look in mailbox:' User2'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

call SetDigestMember(User2'@'Domain)

say ''
say '19. Message from  digest member'
rc = lineout(LogFile, '19. Message from  digest member', )
say ''

MsgFrom = User2'@'Domain
MsgFile = '19.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User2'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from digest member', )
rc = lineout(MsgFile, 'Marker: 19. Message from digest member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 19. Message from  digest member.', )
rc = lineout(LogFile, 'List should receive message.', )
rc = lineout(LogFile, 'Digest file should be updated.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say 'Fifth series -- Message Functions/Closed List'
rc = lineout(LogFile, 'Fifth series -- Message Functions/Closed List', )
Moderated = 1
NoList = 0
SubscribePolicy = 'open'
OpenPosting = 0
call ReWriteCf

say ''
say '20. Message from member'
rc = lineout(LogFile, '20. Message from member', )
say ''

MsgFrom = User'@'Domain
MsgFile = '20.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from member', )
rc = lineout(MsgFile, 'Marker: 20. Message from member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 20. Message from member.', )
rc = lineout(LogFile, 'Moderator should receive approval request.', )
rc = lineout(LogFile, 'Sender should receive acknowledgement.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

call SetDigestMember('') /* no digest member */

say ''
say '21. Message from non-member'
rc = lineout(LogFile, '21. Message from non-member', )
say ''

MsgFile = '21.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User2'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from non-member', )
rc = lineout(MsgFile, 'Marker: 21. Message from non-member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 21. Message from non-member.', )
rc = lineout(LogFile, 'Sender should receive error message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

call SetDigestMember(User2'@'Domain)

say ''
say '22. Message from  digest member'
rc = lineout(LogFile, '22. Message from  digest member', )
say ''

MsgFile = '22.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User2'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from digest member', )
rc = lineout(MsgFile, 'Marker: 22. Message from digest member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 22. Message from  digest member.', )
rc = lineout(LogFile, 'Moderator should receive approval request.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, 'Sender should receive acknowledgement.', )
rc = lineout(LogFile, 'Look in mailbox:' User2'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '23. Message from  digest member, approved'
rc = lineout(LogFile, '23. Message from  digest member, approved', )
say ''

MsgFile = '23.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User2'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'approved: test-pass', )
rc = lineout(MsgFile, 'message from digest member, approved', )
rc = lineout(MsgFile, 'Marker: 23. Message from digest member, approved.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 23. Message from  digest member, approved.', )
rc = lineout(LogFile, 'List should receive message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say 'Sixth series -- Message Functions/Open List/Open Posting'
rc = lineout(LogFile, 'Sixth series -- Message Functions/Open List/Open Posting', )
Moderated = 0
NoList = 0
SubscribePolicy = 'open'
OpenPosting = 1
call ReWriteCf
call SetMember(User'@'Domain)

say ''
say '24. Message from member'
rc = lineout(LogFile, '24. Message from member', )
say ''

MsgFrom = User'@'Domain
MsgFile = '24.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from member', )
rc = lineout(MsgFile, 'Marker: 24. Message from member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 24. Message from member.', )
rc = lineout(LogFile, 'List should receive message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

call SetDigestMember('')
say ''
say '25. Message from non-member'
rc = lineout(LogFile, '25. Message from non-member', )
say ''

MsgFile = '25.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User2'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from non-member', )
rc = lineout(MsgFile, 'Marker: 25. Message from non-member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 25. Message from non-member.', )
rc = lineout(LogFile, 'List should get message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

call SetDigestMember(User2'@'Domain)

say ''
say '26. Message from  digest member'
rc = lineout(LogFile, '26. Message from  digest member', )
say ''

MsgFile = '26.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User2'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from digest member', )
rc = lineout(MsgFile, 'Marker: 26. Message from digest member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 26. Message from  digest member.', )
rc = lineout(LogFile, 'List should receive message.', )
rc = lineout(LogFile, 'Digest file should be updated.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say 'Seventh series -- Message Functions/Closed List/Open Posting'
rc = lineout(LogFile, 'Seventh series -- Message Functions/Closed List/Open Posting', )
Moderated = 1
NoList = 0
SubscribePolicy = 'open'
OpenPosting = 1
call ReWriteCf

say ''
say '27. Message from member'
rc = lineout(LogFile, '27. Message from member', )
say ''

MsgFrom = User'@'Domain
MsgFile = '27.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from member', )
rc = lineout(MsgFile, 'Marker: 27. Message from member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 27. Message from member.', )
rc = lineout(LogFile, 'Moderator should receive approval request.', )
rc = lineout(LogFile, 'Sender should receive acknowledgement.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

call SetDigestMember('')
say ''
say '28. Message from non-member'
rc = lineout(LogFile, '28. Message from non-member', )
say ''

MsgFile = '28.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

call SetDigestMember('') /* no digest member */

MsgFrom = User2'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from non-member', )
rc = lineout(MsgFile, 'Marker: 28. Message from non-member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 28. Message from non-member.', )
rc = lineout(LogFile, 'Sender should receive acknowledgement.', )
rc = lineout(LogFile, 'Look in mailbox:' User2'@'Domain, )
rc = lineout(LogFile, 'Moderator should receive approval request.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

call SetDigestMember(User2'@'Domain)

say ''
say '29. Message from digest member'
rc = lineout(LogFile, '29. Message from digest member', )
say ''

MsgFile = '29.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User2'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'message from digest member', )
rc = lineout(MsgFile, 'Marker: 29. Message from digest member.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 29. Message from  digest member.', )
rc = lineout(LogFile, 'Moderator should receive approval request.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, 'Sender should receive acknowledgement.', )
rc = lineout(LogFile, 'Look in mailbox:' User2'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

say ''
say '30. Message from member, approved'
rc = lineout(LogFile, '30. Message from member, approved', )
say ''

MsgFile = '30.tmp'
rc = stream(MsgFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to open temporary file:' MsgFile
  exit
  end

MsgFrom = User'@'Domain
MsgTo = ListName
call WriteHeaders
rc = lineout(MsgFile, 'approved: test-pass', )
rc = lineout(MsgFile, 'message from member, approved', )
rc = lineout(MsgFile, 'Marker: 30. Message from member, approved.', )
rc = lineout(MsgFile, '', )
rc = stream(MsgFile, 'c', 'close')

rc = lineout(LogFile, 'Marker: 30. Message from member, approved.', )
rc = lineout(LogFile, 'List should receive message.', )
rc = lineout(LogFile, 'Look in mailbox:' User'@'Domain, )
rc = lineout(LogFile, '', )

/* Call Steward */
start '/C /B' Steward ListName MsgFile
rc = SysSleep(10)

/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */
/* -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ */

rc = stream(LogFile, 'c', 'close')

say ''
say 'Look at the file' LogFile 'for details on the tests conducted'
say 'and the expected results.'

Junk = directory(OldDir)

exit

/* ------------------------------------------------------------------ */

ReWriteCf:

rc = SysFileDelete(ListCf)
rc = stream(ListCf, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to rewrite' ListCf 'file.'
  exit
  end

rc = lineout(ListCf, 'AdminPassword = test-pass', )
rc = lineout(ListCf, 'ListOwner =' User'@'Domain, )
rc = lineout(ListCf, 'ApprovePassword = test-pass', )
rc = lineout(ListCf, 'DoArchive = 0', )
rc = lineout(ListCf, 'Moderated =' Moderated, )
rc = lineout(ListCf, 'NoList =' NoList, )
rc = lineout(ListCf, 'Precedence = 1', )
rc = lineout(ListCf, 'ListHeader = 1', )
rc = lineout(ListCf, 'DoDigest = 1', )
rc = lineout(ListCf, 'DigestVolume = 1', )
rc = lineout(ListCf, 'DigestIssue = 1', )
rc = lineout(ListCf, 'DigestName = Test Digest', )
rc = lineout(ListCf, 'DigestRmHeader = 1', )
rc = lineout(ListCf, 'DigestFronter = fronter.txt', )
rc = lineout(ListCf, 'DigestFooter = footer.txt', )
rc = lineout(ListCf, 'DigestSubs = 1', )
rc = lineout(ListCf, 'DigestDelete = 1', )
rc = lineout(ListCf, 'SubscribePolicy =' SubscribePolicy, )
rc = lineout(ListCf, 'ReplyTo =' ListName'@'Domain, )
rc = lineout(ListCf, 'SubjectPrefix = Test-List:', )
rc = lineout(ListCf, 'OpenPosting =' OpenPosting, )
rc = lineout(ListCf, 'WelcomeFile = welcome.txt', )
rc = lineout(ListCf, 'CaseInsensitive = 0', )
rc = stream(ListCf, 'c', 'close')

return


/* ------------------------------------------------------------------ */

WriteHeaders:

TimeZone = value( 'TZ', , Env)
TmpTime = time('N')
DayOfWeek = date('W')
DayOfWeek = left(DayOfWeek, 3)
TmpDate = date('N')
rc = lineout(MsgFile, 'Date:' DayOfWeek',' TmpDate TmpTime TimeZone, )
rc = lineout(MsgFile, 'From:' MsgFrom, )
rc = lineout(MsgFile, 'To:' MsgTo, )
rc = lineout(MsgFile, 'Subject:' MsgFile, )
rc = lineout(MsgFile, '', )

return

/* ------------------------------------------------------------------ */

Init:

call ReadMasterCf
/* change to the Steward Home Directory */
ListCf = ListDir'\'ListName'\'ListName'.cf'
MemberFile = ListDir'\'ListName'\'ListName
rc = SysFileDelete(MemberFile)

return

/* ------------------------------------------------------------------ */

SetMember:

parse arg Member
rc = SysFileDelete(MemberFile)
rc = stream(MemberFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to write member file.'
  return
  end
rc = lineout(MemberFile, Member)
rc = stream(MemberFile, 'c', 'close')

return

/* ------------------------------------------------------------------ */

SetDigestMember:

parse arg Member

DigestFile = MemberFile'.digest'
rc = SysFileDelete(DigestFile)
rc = stream(DigestFile, 'c', 'open write')
if rc <> 'READY:' then do
  say 'Unable to write member file.'
  return
  end
rc = lineout(DigestFile, Member)
rc = stream(DigestFile, 'c', 'close')

return

/* ------------------------------------------------------------------ */
/*
 * Read the master configuration file
 *
 */

ReadMasterCf: 

/* Find out where the configuration file should be */
/* StewardCf = value('steward_cf',,Env) */
 StewardCf = value('steward_cf_test',,Env)

/* If its not defined then assume wherever we are */
if StewardCf = '' then do
  StewardCf = '.'
  end

FileName = StewardCf'\steward.cf'

rc = LockOpen(FileName 'READ')  /* open the file locking it */
if rc = FALSE then
  return FALSE                   /* return FALSE if cannot open */

/* now read the configuration file */
do while lines(FileName) <> 0         /* until end of file */
  Line = linein(FileName)             /* get a line of the file */
  parse var Line Line '#' Comment     /* separate out any comments */
  if Line <> '' then do               /* if not null */
    parse var Line Key '=' Val        /* find the key and value */
    if Key <> '' then do
      Val = strip(Val, 'B', ' ')      /* remove any blanks */
      Key = strip(Key, 'B', ' ')
      select
        when Key = 'HomeDir' then
          HomeDir = Val
        when Key = 'LogDir' then
          LogDir = Val
        when Key = 'ListDir' then
          ListDir = Val
        when Key = 'Mailer' then
          Mailer = Val
        when Key = 'WhereAmI' then
          WhereAmI = Val
        when Key = 'WhoAmI' then
          WhoAmI = Val
        when Key = 'WhoAmIOwner' then
          WhoAmIOwner = Val
        when Key = 'MasterPassword' then
          MasterPassword = Val
        otherwise nop
        end   /* select */
      end     /* if Key <> '' */
    end       /* if Line <> '' */

  Key = ''

end /* end do while */

rc = LockClose(FileName)

if Debug = TRUE then say 'Steward.cf file read.'

return TRUE

/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */

